/*
 * This file is part of gshutdown.
 *
 * Copyright (c) 2006-2009, GShutdown Team <gshutdown@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with This program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <signal.h>
#include "action.h"
#include "misc.h"
#include "gui.h"
#include "values.h"
#include "gdm.h"
#include "dbus.h"

/* Run the action now ! */
static gboolean action_run_command(Action *action);

/*
 * Constructor.
 */
Action *action_new()
{
    Action *ptr = g_malloc(sizeof(Action));
    ptr->type = ACTION_TYPE_NONE;
    ptr->command = NULL;
    return ptr;
}

/* 
 * Destructor.
 */
void action_destroy(Action *action)
{
    if(action->command != NULL)
        g_free(action->command);
    else
        g_free(action);
}

/*
 * Read the action type (see ActionType
 * in action.h)
 */
ActionType action_get_type(Action *action)
{
    return action->type;
}

/*
 * Modity the action type (see ActionType
 * in action.h)
 */
void action_set_type(Action *action, ActionType type)
{
    action->type = type;
}

/*
 * Change the custom command.
 *
 * NB: "command" string is duplicated with g_strdup.
 */
void action_set_command(Action *action, const gchar *command)
{
    if(action->command != NULL)
        g_free(action->command);

    action->command = g_strdup(command);
}

/*
 * Return the custom command.
 *
 * Warning: the returned pointer is internal, DO
 *          NOT FREE IT!
 */
const gchar *action_get_command(Action *action)
{
    return (action->command) ? action->command : "";
}

/*
 * Kill a session by using the PID in the environment
 * variable SESSION_MANAGER.
 */
void kill_session()
{
    const gchar *session_manager = g_getenv("SESSION_MANAGER");
    if(session_manager) {
        gchar **splited = g_strsplit(session_manager, "/", 0);

        if(splited != NULL) {
            guint length = g_strv_length(splited);
            if(length > 0) {
                pid_t pid = atoi(splited[length - 1]);
                if(pid != 0)
                    kill(pid, SIGQUIT);
            }
            g_strfreev(splited);
        }
    }
}

/*
 * Run the actual action.
 *
 * Return TRUE on error.
 */
gboolean action_run(Action *action)
{
    /* Command before action */
    if(values.command_before_action_enabled == TRUE)
        run_command(values.command_before_action);

    switch(action_get_type(action)) {
        case ACTION_TYPE_SHUTDOWN:
            return launch_shutdown(action);
        case ACTION_TYPE_RESTART:
            return launch_reboot(action);
        case ACTION_TYPE_LOCK: 
            return launch_lock(action);
        case ACTION_TYPE_LOGOUT:
            return launch_logout(action);
        case ACTION_TYPE_HIBERNATE:
            return launch_hibernate(action);
        case ACTION_TYPE_CUSTOM:
            return launch_custom(action);
        case ACTION_TYPE_NONE:
            return FALSE;
    }

    return TRUE;
}

/*
 * Run the custom command (action->command).
 *
 * Return TRUE on error.
 */
static gboolean action_run_command(Action *action)
{
    if(action->command == NULL) 
        return TRUE;

    return run_command(action->command);
}


/*
 * launch action
 */
gboolean launch_shutdown(Action *action)
{
    if(values.action_method == ACTION_METHOD_CUSTOM_COMMAND) {
        return action_run_command(action);
    } else {
        if(dbus_action("org.freedesktop.PowerManagement", "/org/freedesktop/PowerManagement", "org.freedesktop.PowerManagement", "Shutdown")) {
            Desktop current_desktop = values.detected_desktop;
            DisplayManager current_display_manager =  values.detected_display_manager;

            /* Method = action selected manually */
            if(values.action_method == ACTION_METHOD_MANUAL) {
                if(values.selected_desktop != DESKTOP_AUTO)
                    current_desktop = values.selected_desktop;

                if(values.selected_display_manager != DISPLAY_MANAGER_AUTO)
                    current_display_manager = values.selected_display_manager;
            }

            /* Not a root command ? Quit here, it's not useful to talk with the
             * session manager now. */

            /* WARNING: After this line, its important to use
             * current_display_manager and current_desktop */
            
            if(current_desktop == DESKTOP_NONE || 
               current_display_manager == DISPLAY_MANAGER_NONE)
                return TRUE;
            
            /* GDM */
            if(current_display_manager == DISPLAY_MANAGER_GDM) {
                run_command("gdmflexiserver -a --command=\"SET_LOGOUT_ACTION HALT\"");
            } 
            /* KDM */
            else if(current_display_manager == DISPLAY_MANAGER_KDM) {
                run_command("kdmctl shutdown halt forcenow");
            }
            
            return launch_logout(NULL);
        }
        else
            return FALSE;
    }
}

gboolean launch_reboot(Action *action)
{
    /* TODO : Ajouter test pour savoir s'il faut utiliser dbus ou GDM/KDM */
    if(values.action_method == ACTION_METHOD_CUSTOM_COMMAND) {
        return action_run_command(action);
    } else {
        if(dbus_action("org.freedesktop.PowerManagement", "/org/freedesktop/PowerManagement", "org.freedesktop.PowerManagement", "Reboot")) {
            Desktop current_desktop = values.detected_desktop;
            DisplayManager current_display_manager =  values.detected_display_manager;

            /* Method = action selected manually */
            if(values.action_method == ACTION_METHOD_MANUAL) {
                if(values.selected_desktop != DESKTOP_AUTO)
                    current_desktop = values.selected_desktop;

                if(values.selected_display_manager != DISPLAY_MANAGER_AUTO)
                    current_display_manager = values.selected_display_manager;
            }

            /* Not a root command ? Quit here, it's not useful to talk with the
             * session manager now. */

            /* WARNING: After this line, its important to use
             * current_display_manager and current_desktop */
            
            if(current_desktop == DESKTOP_NONE || 
               current_display_manager == DISPLAY_MANAGER_NONE)
                return TRUE;
            
            /* GDM */
            if(current_display_manager == DISPLAY_MANAGER_GDM) {
                run_command("gdmflexiserver -a --command=\"SET_LOGOUT_ACTION REBOOT\"");
            } 
            /* KDM */
            else if(current_display_manager == DISPLAY_MANAGER_KDM) {
                run_command("kdmctl shutdown reboot forcenow");
            }
            
            return launch_logout(NULL);
        }
        else
            return FALSE;
    }
}

gboolean launch_lock(Action *action)
{
    Desktop current_desktop = values.detected_desktop;

    /* GNOME */
    if(current_desktop == DESKTOP_GNOME)
        return dbus_action("org.gnome.ScreenSaver", "/", "org.gnome.ScreenSaver", "Lock");
    /* KDE 4 */
    if(current_desktop == DESKTOP_KDE)
        return dbus_action("org.freedesktop.ScreenSaver", "/ScreenSaver", "org.freedesktop.ScreenSaver", "Lock");

    return TRUE;
}

gboolean launch_hibernate(Action *action)
{
    return dbus_action("org.freedesktop.PowerManagement", "/org/freedesktop/PowerManagement", "org.freedesktop.PowerManagement", "Hibernate");
}

gboolean launch_logout(Action *action)
{
    Desktop current_desktop = values.detected_desktop;

    /* GNOME */
    if(current_desktop == DESKTOP_GNOME) 
        return run_command("gnome-session-save --kill --silent");
    /* KDE */
    else if(current_desktop == DESKTOP_KDE)
	{
		run_command("kdeinit_shutdown");
        	return run_command("kdeinit4_shutdown");
	}
    /* XFCE */
    else if(current_desktop == DESKTOP_XFCE)
        kill_session();


    return TRUE;
}

gboolean launch_custom(Action *action)
{
    return run_command(action->command);   
}


/* vim:cindent:et:sw=4:ts=4:sts=4:tw=78:fenc=utf-8
 */
